#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include "M6809.h"
#include <signal.h>
#include <sys/time.h>

extern unsigned char ROM[0x5000];
extern unsigned char VIDEO[0x2000];
extern unsigned char ATRIB[0x2000];
extern void initvideo();
extern unsigned char key[64];
extern void loadhex(char *);
extern void opentape(char *);
extern void loadsna(char *);
extern void opentout(char *fn);
extern void print_help();
void init_keytbl(char *fn);

#ifndef SOUND
void alarma(){}
void init_alarm()
{
    struct itimerval t0,t1;
    struct sigaction act0,act1;

    fprintf(stderr,"main.init_alarm: Starting 50-Hz timing event\n");
    t0.it_interval.tv_sec=0;
    t0.it_interval.tv_usec=20000;
    t0.it_value=t0.it_interval;

    act0.sa_handler=alarma;
    act0.sa_flags=SA_RESTART;

    sigaction(SIGALRM,&act0,&act1);
    setitimer(ITIMER_REAL,&t0,&t1);
}
#endif

main(int argc, char **argv)
{
	int fd,i;
	char *sna=NULL,*romf=ROMFILE,*cf=NULL;

        // Clean up the cartridge area, just in case...
        for (i=0;i<0x1000;i++) ROM[i]=0xff;

        fprintf(stderr,"\n");

	for (i=1;i<argc;i++) {
		if (*argv[i]=='-') {
			switch(*(argv[i]+1)) {
			case 't':	opentape(argv[++i]);
					break;
                        case 'o':       opentout(argv[++i]);
                                        break;
			case 'x':	loadhex(argv[++i]);
					break;
                        case 'r':       romf=argv[++i]; break;
                        case 'c':       cf=argv[++i]; break;
                        case 'k':       init_keytbl(argv[++i]); break;
			case 'h':
			default:
				fprintf(stderr,"Use: xmo5 [options] [Snapshot_file]\n");
				fprintf(stderr," -h			This help\n");
				fprintf(stderr," -t file(.k7)		Attach \"file.k7\" to tape input\n");
                                fprintf(stderr," -o file(.k7)		Attach \"file.k7\" to tape output\n");
				fprintf(stderr," -x file(.s19,.ihex)	Load hexadecimal file to RAM\n");
                                fprintf(stderr," -r file                Load ROM image from \"file\"\n");
                                fprintf(stderr," -c file		Load cartridge image from \"file\"\n");
                                fprintf(stderr," -k file                Load keymap from \"file\"\n");
				print_help();
				exit(0);

			}
		} else {
			sna=argv[i];
		}
	}

        if ((fd=open(romf,O_RDONLY))==-1) {
		perror("open: Main_ROM ");
		exit(1);
	}
	read (fd,&ROM[0x1000],0x4000);
	close (fd);
	fprintf(stderr,"main: ROM Image \"%s\" loaded\n",romf);
        if (cf) {
            if ((fd=open(cf,O_RDONLY))==-1) {
		perror("open: Cartridge_ROM");
		exit(1);
	    }
	    i=read (fd,ROM,0x4000);
	    close (fd);
	    fprintf(stderr,"main: Cartridge Image \"%s\", %d bytes, loaded\n",cf,i);
        }

	initvideo();

#ifdef	SOUND
	init_sound();
        fprintf(stderr,"main: Using sound device timing\n");
#else
        init_alarm();
#endif

	for (i=0;i<64;i++) key[i]=0;	/* Initial state of keyboard */

	if (sna==NULL) {
		m6809_reset();
		fprintf(stderr,"main: Starting Emulation from RESET\n");
	} else loadsna(sna);

	m6809_execute();

	printf("ME PAR / I'm STOPPED / Je suis detenue\n");

}
