#include "cpuintrf.h"
#include "M6809.h"
#include "keys.h"
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>

extern unsigned long m6809_cpu_clk;

int ticcounter=0;               /* sampling counter */

unsigned char RAM[0x8000];	/* 32 Kb RAM at offset 0x2000 */
unsigned char ROM[0x5000];	/* 16 Kb ROM (+4K) at offset 0xB000 */

#define SYS_PIA0  0xA7C0
#define SYS_PIA1  0xA7C1
#define SYS_PIA2  0xA7C2
#define SYS_PIA3  0xA7C3

#define AUX_PIA0  0xA7CC
#define AUX_PIA1  0xA7CD
#define AUX_PIA2  0xA7CE
#define AUX_PIA3  0xA7CF

#define PRT_PIA0  0xA7E0
#define PRT_PIA1  0xA7E1
#define PRT_PIA2  0xA7E2
#define PRT_PIA3  0xA7E3

#define GATE_ARR0 0xA7E4
#define GATE_ARR1 0xA7E5
#define GATE_ARR2 0xA7E6
#define GATE_ARR3 0xA7E7


struct {
	unsigned char sys_pia_pa;		/* Main system SYS_PIA */
	unsigned char sys_pia_pac;		/* Partially emulated */
	unsigned char sys_pia_pb;
	unsigned char sys_pia_pbc;

/*	unsigned char aux_pia_pa;	// Aux. PIA (Joy stics)
	unsigned char aux_pia_pac;
	unsigned char aux_pia_pb;
	unsigned char aux_pia_pbc;

        unsigned char prt_pia_pa;	// Printer PIA
	unsigned char prt_pia_pac;
	unsigned char prt_pia_pb;
	unsigned char prt_pia_pbc; */
} io;

#define XOFF 12                 // Light-pen offset (due to delays)
unsigned int gate_array_cnt;    // Gate array counter (pixel counter)
unsigned char DAC=0;            // Aux PIA DAC (6-bit audio)

unsigned char VIDEO[0x2000];	/* 8 Kb pixels at offset 0 */
unsigned char ATRIB[0x2000];	/* 8 Kb colors at offset 0 */

unsigned char key[64];		/* Keyboard Array */
unsigned int colort[16];	/* Color Table for pixels */

extern  char *XImageData;
extern  int depth;
extern  unsigned int pen_x,pen_y,pen_b;
extern  unsigned int joystick;
extern  int tostatus;

#ifdef	SOUND
extern void sound_decimate(int in);
#endif

extern void tapeout(unsigned char data);
//extern void bitout(int data);

int     dirtyvideo=0;		/* interfaz con E/S X11 */
int     keycommand=NOP;
int     turbo=0;
int     tout_edges=0;

int cpu_readmem(int address)
{
	int nkey;
	if (address<0x2000) {
		if (io.sys_pia_pa&1) return VIDEO[address];
		else return ATRIB[address];
	}
	if (address<0xA000) return RAM[address-0x2000];
	if (address>=0xB000) return ROM[address-0xB000];

        switch(address) {
        //------------- SYSTEM PIA --------------
	case SYS_PIA0:  return io.sys_pia_pa;
	case SYS_PIA1:
                        // Keyboard emulation
                        // PB1-PB6 select the key
                        // PB7 is 0 when the key is pressed (1 if not)
		        nkey=(io.sys_pia_pb>>1)&63;
		        if (key[nkey]) io.sys_pia_pb &=0x7f;
                        else io.sys_pia_pb |=0x80;
		        return io.sys_pia_pb;
                        // Interrupt flags are always 1
        case SYS_PIA2:  return io.sys_pia_pac|0x80;
        case SYS_PIA3:  return io.sys_pia_pbc|0x80;
        //--------------- AUX PIA ------------------
        case AUX_PIA0:  return joystick&0xff;
        case AUX_PIA1:  return ((joystick>>8)&0xc0)|DAC;
        //----------- GATE ARRAY counters and flags --------------------
        case GATE_ARR0: return (gate_array_cnt>>8)&0xff;
        case GATE_ARR1: return gate_array_cnt&0xff;
                        // Horizontal field can be off due to pen delays
        case GATE_ARR2: return ((pen_x+XOFF)<320)?0x40:0x80;
                        // Vertical field bit 1=screen 0=border
        case GATE_ARR3: return (ticcounter>9 && ticcounter<39)?0x80:0;
        //--------------------------------------------------------------
        default:        return 0xff;
        }
}

void cpu_writemem(int address,int data)
{
	int i,j,x,y;
        unsigned int *pi,bg,fg,t;
	unsigned short *ps;
	unsigned char *pc;

#ifdef SOUND
        if (io.sys_pia_pac&0x8 && !turbo)
            sound_decimate(((io.sys_pia_pb&1)<<6)+DAC);
#endif
        //-------------- Video RAM (pixels & color) --------------------
        if (address<0x2000) {
	    if (io.sys_pia_pa & 1) {
		if (VIDEO[address]!=data) {
			VIDEO[address]=data;
			dirtyvideo=1;
		}
	    } else {
		if (ATRIB[address]!=data) {
			ATRIB[address]=data;
			dirtyvideo=1;
		}
		data=VIDEO[address];
	    }
            //---------- If visible we have to draw it ----------
            if (address<8000) {
		bg=ATRIB[address] & 0xf;
		fg=(ATRIB[address] >>4) & 0xf;
		bg=colort[bg]; fg=colort[fg];

                pi=(unsigned int *)XImageData;
                ps=(unsigned short *)XImageData;
		pc=(unsigned char *)XImageData;

#ifdef  SCALE2
		j=address<<3;
		x=j%320; y=j/320;
		j=(x<<1) + 1280*y;
#else
		j=address<<3;
#endif
                pi+=j; ps+=j; pc+=j;
		for (i=8;i;i--) {
#ifdef	SCALE2
			t=(data & 0x80)?fg:bg;
			if (depth==32) {
				*pi=t;
				*(pi+640)=t; pi++;
				*pi=t;
				*(pi+640)=t; pi++;

			}else if (depth==16) {
				*ps=t;
				*(ps+640)=t; ps++;
				*ps=t;
				*(ps+640)=t; ps++;

			} else if (depth==8) {
				*pc=t;
				*(pc+640)=t; pc++;
				*pc=t;
				*(pc+640)=t; pc++;
			}
#else
                        if (depth==32) *pi++=(data & 0x80)?fg:bg;
                        else if (depth==16) *ps++=(data & 0x80)?fg:bg;
			else if (depth==8) *pc++=(data & 0x80)?fg:bg;
#endif
			data<<=1;
		}
            }
            return;
	}
        //--------------------- Regular RAM -----------------------
	if (address<0xA000) { RAM[address-0x2000]=data; return;}

        switch(address){
        //----------------------- System PIA ----------------------
        case SYS_PIA0:
                // We do not change inputs
                // PA0          pixel RAM/color RAM
                // PA1-PA4      Border color (not used)
                // PA5          ligth pen button (input)
                // PA6          cassette output
                // PA7          cassette input
		data&=0x5F;	        //0101.1111
		io.sys_pia_pa&=0xA0;	//1010.0000
		io.sys_pia_pa|=data;
                // If motor is on emulate cassette output (PA6)
                if (!(io.sys_pia_pac&8)) tapeout(io.sys_pia_pa&0x40);
                break;
	case SYS_PIA1:
                // Keyboard control and audio output (PB0)
		io.sys_pia_pb=data; break;
	case SYS_PIA2:
                // This register is used for controlling FIRQ interrupts
                // and the motor of the cassette
                // bit 0: FIRQ enable (ligth-pen)
                // bit 3: CA2 control (motor ON if 0)
                // bit 7: FIRQ flag (always on)

                // Cassette motor turned OFF => clear tape stuff
                if ((io.sys_pia_pac&8)==0 && (data&8)) {
                    tapeout(1); // Flush last bit to tape
                    tout_edges=0;
                    tostatus=0;
                }
                io.sys_pia_pac=data;
                break;
	case SYS_PIA3:
                io.sys_pia_pbc=data;
                break;
        case AUX_PIA1:
                DAC=data&0x3f;
                break;
        }
}

/***************************************************************************/

extern  void video_uptade();
extern  void print_help();
	void save_sna();
	void load_sna();
extern  void load_hex();

extern  int newbit();
extern  void tape_attach();
extern  void tape_restart();
extern  void tout_attach();
extern  void flushtout();
extern  void tape_rewind();

// This function is called  2400 times/sec (simulated time)
// It provides
// - screen and keyboard refresh (X11 interface)
// - tape input emulation
// - light-pen emulation
// - Interrupt posting to 6809

int cpu_interrupt(void)
{
	static short edgeFF=0;
	static int nb;
        int i;

        // If motor is ON and there is no activity on tape_output
        // then tape_input is emulated
        if ((io.sys_pia_pac&8)==0 && tout_edges<8) {
            if (!edgeFF)
		if (nb=newbit()) io.sys_pia_pa|=0x80; else io.sys_pia_pa&=0x7f;
	    else
		if (nb<2) io.sys_pia_pa^=0x80;
        } else tape_restart();
	edgeFF^=1;

        // Video/keyboard update and Vertical retrace interrupt post
        // called every 20 ms (50 Hz) or 48 emulator tics.
        if ((++ticcounter)==48) {
		ticcounter=0;
#ifndef	SOUND
		if (io.sys_pia_pac&0x8 && !turbo) pause();
#endif
		switch (keycommand) {
		case NOP:
				break;
		case HELP:
				print_help(); break;
		case SAVESTATUS:
				save_sna(); break;
		case LOADSTATUS:
				load_sna(); break;
		case ATTACHTAPE:
				tape_attach(); break;
                case ATTACHTOUT:
                                tout_attach(); break;
                case REWINDTAPE:
				tape_rewind(); break;
		case LOADHEX:
				load_hex(); break;
		case RESET:
				m6809_reset();
				fprintf(stderr,"mo5: 6809 RESET\n");
				break;
		case TURBO:	turbo^=1;
				fprintf(stderr,"mo5: TURBO: %s\n",(turbo)?"ON":"OFF");
				break;
		case EXITEMU:
                                flushtout();
				exit(0);
		}
		keycommand=NOP;
                video_update();
		return INT_IRQ;
	}

        // Light-pen emulation
        // Pen button is connected to SYS_PIA PA5
        // During one frame the ligth-pen must generate at least
        // 2 FIRQs. The FIRQ routine reads the Gate-Array counters.
        // The counters must reflect the time passed between successive
        // FIRQs (idealy 1 scanline or 64 us or 320 pixels).
        // FIRQs are enabled by SYS_PIA PAC0 (control, portA)

        if (pen_b) io.sys_pia_pa|=0x20; else io.sys_pia_pa&=~0x20;
        if (ticcounter>24 && ticcounter<28) {
            i=pen_y+ticcounter-26;
            if (i<0 || i>=200) return 0;
            gate_array_cnt=i*320+pen_x+XOFF;
            if (io.sys_pia_pac&1) return INT_FIRQ;
        }
        return 0;
}

/*********************************************************************/
void save_sna()
{
	char fn[256];
	int fd,i;
	m6809_Regs R;

	printf("mo5: Snapshot File name :"); fflush(stdout);
	gets(fn);
	if ((fd=creat(fn,0666))==-1) {
		perror("mo5.save_sna");
		return;
	}
	i=0xBEBECAFE;
	write(fd,&i,4);
	m6809_GetRegs(&R);
	write(fd,&R,sizeof(m6809_Regs));
	write(fd,&io,sizeof(io));
	write(fd,VIDEO,0x2000);
	write(fd,ATRIB,0x2000);
	write(fd,RAM,0x8000);
	close(fd);
	fprintf(stderr,"mo5.save_sna: Snapshot File Saved\n");
}

void loadsna(char *fn)
{
	int fd,i;
	m6809_Regs R;

	if ((fd=open(fn,O_RDONLY))==-1) {
		perror("mo5.loadsna");
		return;
	}
	read(fd,&i,4);
	if (i!=0xBEBECAFE) {
		fprintf(stderr,"mo5.loadsna: Bad Magic Number\n");
		close(fd);
		return;
	}
	read(fd,&R,sizeof(m6809_Regs));
	m6809_SetRegs(&R);
	read(fd,&io,sizeof(io));
	read(fd,VIDEO,0x2000);
	read(fd,ATRIB,0x2000);
	read(fd,RAM,0x8000);
	close(fd);
	
	fd=io.sys_pia_pa;
	io.sys_pia_pa=1;
	for (i=0;i<8000;i++) cpu_writemem(i,VIDEO[i]);
	io.sys_pia_pa=fd;
	dirtyvideo=1;
	
	fprintf(stderr,"mo5.loadsna: Snapshot File Loaded\n");
}

void load_sna()
{
	char fn[256];
	printf("mo5: Snapshot File to load :"); fflush(stdout);
	gets(fn);
	loadsna(fn);
}

