#ifdef	SOUND

#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <linux/soundcard.h>


extern unsigned long m6809_cpu_clk;

int soundfd;

void init_sound()
{
	int i;

        if ((soundfd=open("/dev/dsp", O_WRONLY)) == -1) {
		perror("Sound.OPEN");
                exit(1);
        }

        // Use a small fragment to avoid long sound delays
        i=0x00030007;
        if (ioctl(soundfd, SNDCTL_DSP_SETFRAGMENT, &i)) {
                fprintf(stderr, "Cannot set fragment size\n");
        }
        i=8;
        if (ioctl(soundfd, SNDCTL_DSP_SAMPLESIZE, &i) == EOF) {
                fprintf(stderr, "Cannot set %d bits/sample\n",i);
		exit(1);
        }
        i=0;
        if (ioctl(soundfd, SNDCTL_DSP_STEREO, &i) == EOF) {
                fprintf(stderr, "Cannot set %ld channel(s)\n",i+1);
                exit(1);
        }
        i=22050;
        if (ioctl(soundfd, SNDCTL_DSP_SPEED, &i) == EOF) {
                fprintf(stderr, "Cannot set rate %d Hz\n",i);
		exit(1);
        }

	fprintf(stderr,"Sound Device opened at 22050 Hz, 8 bits, Mono\n");
        
}

// Comb Filter + Decimation + Sound writing
// Decimation ratio is 45 (1 MHz -> 22 KHz)

void sound_decimate(int in)
{
    static unsigned int i1=0,i2=0,d1=0,d2=0,nt=0;
    static unsigned long oclk=0;
    unsigned int i,o1,o2;
    unsigned char a;

    i=m6809_cpu_clk-oclk; oclk=m6809_cpu_clk;
    if (i>1000) return; // Do not trust very long time intervals (turbo)

    for (;i;i--) {
        // Integrators
        i1+=in;
        i2+=i1;
        // decimation
        nt++;
        if (nt==45) {
            nt=0;
            // differentiators
            o1=i2-d1;
            o2=o1-d2;
            d1=i2; d2=o1;
            // 8-bit output
            a=o2>>10; // scaling
            write(soundfd,&a,1);
        }
    }
}

#endif
