#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>


int fdtape=0;
int k7bloat=0;

void opentape(char *fn)
{
	int i;
	unsigned char a;
	if ((fdtape=open(fn,O_RDONLY))==-1) {
		perror("open(TapeFile,O_RDONLY)");
		exit(1);
	}

	// Ckeck the .K7 file format
	// First byte is block type (0, 1, or 0xFF) for stripped files
	// First byte is 'D' for Daniel Coulom tape files
	i=read(fdtape,&a,1);	
	if (i!=1) {fprintf(stderr,"Error: empty file (%s)\n",fn); exit(1);}
	if (a!=0 && a!=1 && a!=0xff) k7bloat=1;	
	lseek(fdtape,0,SEEK_SET);

	fprintf(stderr,"Tape_IN File attached : %s (%s k7)\n",fn,(k7bloat)?"bloated":"stripped");
}


/********************  Interface K7 ***********************/

/***** Tape emulator state *****/

enum { INIT=0,GAP,SYNC,MARK1,MARK2,BLKTYPE,BLKCNT,DATA,POST,ENDT };

int tapestate=INIT;	/* Current state */

/***** GAPS & SYNC ******/

#define NGAP0	1000	// 1000 = 6.6 s between programs
#define NGAP1	75	// 75 = 500 ms between header & 1st data block
#define NGAP2	15	// 15 = 100 ms between blocks
#define	NSYNC	10	// 10 = 66  ms preable
#define	NPOST	2	// 2   mote bytes after block (paranoid)


// get the proper byte depending on current state
// return value > 0xff  => No data available

int newbyte()
{
	static int nbytes,type,counter;
#ifdef	DEBUG
	static int nb;
#endif

	unsigned char byte;
        int i;

	switch (tapestate) {

	case INIT:	tapestate++;
				counter=NGAP0;
				return 0x100;

	case GAP:	if (!counter--) {
					counter=NSYNC;
					tapestate++;
				}
				return 0x100;

	case SYNC:	if (!counter--) {
				   if (k7bloat) {
					i=0;
					do {
					    if (read(fdtape,&byte,1)!=1) {tapestate=ENDT; return 0x100;}
					    i<<=8; i|=byte; i&=0xffffff;
					} while (i!=0x013c5a);
				    }
				    tapestate++;
				}
				return 0x01;

	case MARK1:	tapestate++;
				return 0x3c;

	case MARK2:	tapestate++;
				return 0x5a;

	case BLKTYPE:	i=read(fdtape,&byte,1);
					if (i>0) tapestate++; else {tapestate=ENDT; return 0x100;}
					type=byte;
					return byte;

	case BLKCNT:    i=read(fdtape,&byte,1);
					if (i>0) tapestate++; else {tapestate=ENDT; return 0x100;}
					nbytes=byte;
					if (!nbytes) nbytes=0x100;
#ifdef  DEBUG
					nb=nbytes;
#endif
					nbytes--;
					return byte;

	case DATA:	i=read(fdtape,&byte,1);
                if (i<=0) {tapestate=ENDT; return 0x100;}
				nbytes--;
				if (!nbytes) {
					tapestate++;
					counter=NPOST;
				}
				return byte;

	case POST:	if (!(--counter)) {
				tapestate=GAP;
				if (type==0xff) counter=NGAP0;
				else if (type==0) counter=NGAP1;
				else counter=NGAP2;
#ifdef	DEBUG
				if (type==0) fprintf(stderr,"\nTape_IN  Block (type.len) = ");
				fprintf(stderr," %02X.%03X ",type,nb);
				if (type==0xff) fprintf(stderr,"\n");
				fflush(stderr);
#endif

				}
				return 0;
        case ENDT:      return 0x100;
	}

}

void tape_restart()
{
    tapestate=INIT;
}

/******************** BIT-level Interface ***********************/

int newbit()
{
	static int bit=0,byte;
	static int counter=0,qflag=0;

	bit&=1;
	if (!counter) {         // Get one byte
		counter=8;
		byte=newbyte();
		if (byte==0x100) qflag=1; else qflag=0;
	}
	counter--;
	if (!qflag){
		if (!(byte&0x80))	bit^=1;	// NRZI: 0 = change
		byte<<=1;
	} else bit|=2;		// do not toggle output during gaps

	return bit;
}

/******* Interactive control *******/

void tape_attach() {
	char fn[256];

	if(fdtape) close(fdtape);
	printf("Tape: File to Attach :"); fflush(stdout);
	gets(fn);
	opentape(fn);
	tapestate=INIT;
}

void tape_rewind()
{
	lseek(fdtape,0,SEEK_SET);
	tapestate=INIT;
	fprintf(stderr,"tape: Tape Rewinded\n");
}

