#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#define  XK_MISCELLANY
#include <X11/keysymdef.h>
#include "../keys.h"

Display *mydisplay;
Window mywindow;
GC mygc;
XEvent myevent;
XSizeHints myhint;
XWMHints mywmhint;
int myscreen;
char *title="Keyborad Map Editor";
unsigned long colf,colb;

void initX()
{
    mydisplay = XOpenDisplay("");
    if(mydisplay==NULL) {fprintf(stderr,"\033[2J\033[HCan't connect to %s\n",
			XDisplayName("")); getchar(); return;}
    myscreen = DefaultScreen(mydisplay);
    colf = WhitePixel(mydisplay,myscreen);
    colb = BlackPixel(mydisplay,myscreen);
    myhint.x=200;myhint.y=0;
    myhint.width=320;myhint.height=200;
    myhint.flags=PPosition|PSize;
    mywmhint.flags=InputHint;
    mywmhint.input=True;

    mywindow=XCreateSimpleWindow(mydisplay,
	DefaultRootWindow(mydisplay),
	myhint.x,myhint.y,myhint.width,myhint.height,
	5,colf,colb);
    XSetStandardProperties(mydisplay,mywindow,title,title,
    	None,NULL,0,&myhint);
    XSetWMHints(mydisplay,mywindow,&mywmhint);

    mygc=XCreateGC(mydisplay,mywindow,0,0);
    XSetBackground(mydisplay,mygc,colb);
    XSetForeground(mydisplay,mygc,colf);

    XSelectInput(mydisplay,mywindow,
	ButtonPressMask|KeyPressMask|KeyReleaseMask|ExposureMask);

    XMapRaised(mydisplay,mywindow);
    //cmap=DefaultColormap(mydisplay,myscreen);

    XClearWindow(mydisplay,mywindow);
    XFlush(mydisplay);
}

typedef struct {
    char *kname;
    unsigned short code;
} keyentry;

keyentry mo5e[]={
"STOP",0x37,"1",0x2f,"2",0x27,"3",0x1f,"4",0x17,"5",0x0f,"6",0x07,
"7",0x06,"8",0x0e,"9",0x16,"0",0x1e,
"Q",0x2d,"W",0x25,"E",0x1d,"R",0x15,"T",0x0d,"Y",0x05,"U",0x04,
"I",0x0c,"O",0x14,"P",0x1c,"+",0x2e,
"A",0x2b,"S",0x23,"D",0x1b,"F",0x13,"G",0x0b,"H",0x03,"J",0x02,
"K",0x0a,"L",0x12,
"Z",0x30,"X",0x28,"C",0x32,"V",0x2a,"B",0x22,"N",0x00,"M",0x08,
", (comma)",0x10,". (dot)",0x18,"- (minus)",0x26,"/ (divide)",0x24,
"*",0x2c,"(space)",0x20,"(shift)",0x38,"(enter)",0x34,"(cntrl)",0x35,
"(backspace)",0x29,"(accent)",0x36,"@",0x1a,"(cls)",0x33,
"(home)",0x11,"(insert)",0x09,"(delete)",0x01,"(basic)",0x39,
"(cursor_up)",0x31,"(cursor_down)",0x21,"(cursor_left)",0x29,
"(cursor_right)",0x19,
"emulator_HELP           (F1)" ,HELP,
"emulator_SAVESNA        (F2)" ,SAVESTATUS,
"emulator_LOADSNA        (F3)" ,LOADSTATUS,
"emulator_ATTACHTAPE_IN  (F4)" ,ATTACHTAPE,
"emulator_REWINDTAPE_IN  (F5)" ,REWINDTAPE,
"emulator_LOADHEX        (F6)" ,LOADHEX,
"emulator_ATTACHTAPE_OUT (F7)" ,ATTACHTOUT,
"emulator_TURBO          (F8)" ,TURBO,
"emulator_EXITEMU        (F10)",EXITEMU,
"emulator_RESET          (F12)",RESET,
"JOYSTICK 1 UP    ",JOYUP,
"JOYSTICK 1 DOWN  ",JOYDOWN,
"JOYSTICK 1 LEFT  ",JOYLEFT,
"JOYSTICK 1 RIGHT ",JOYRIGHT,
"JOYSTICK 1 BUTT  ",JOYBUTT,

NULL,0xff,
};



main(int argc,char **argv)
{
    int i,j,ik=0;
    unsigned char buf[256];
    unsigned short keytab[256];
    keyentry *pk=mo5e;
    FILE *fp;

    if (argc<2) {
        printf("Usage: %s keymap_file\n",argv[0]);
        exit(0);
    }
    sleep(1);
    initX();

    for (i=0;i<256;i++) keytab[i]=NOP;
    for (i=0;i<24;i++) printf("\n");
    printf("Key/button> %s ",pk[ik].kname); fflush(stdout);
    for (;;) {
        if (!XCheckWindowEvent(mydisplay,mywindow,-1,&myevent)){
            usleep(20000);
            continue;
        }

        switch(myevent.type){
        case Expose:
            sprintf(buf,"KEYMAP EDITOR");
            XDrawImageString(mydisplay,mywindow,mygc,160-3*strlen(buf),50,buf,strlen(buf));
            sprintf(buf,"Left Mouse Button: Go Back one entry");
            XDrawImageString(mydisplay,mywindow,mygc,160-3*strlen(buf),100,buf,strlen(buf));
            sprintf(buf,"Right Mouse Button: Finish");
            XDrawImageString(mydisplay,mywindow,mygc,160-3*strlen(buf),150,buf,strlen(buf));

        case KeyPress:
            j=myevent.xkey.keycode;
            if (j==0) break;
            if (j>255) {
                printf("Error: key code >255\n");
                break;
            }
            printf(" -> 0x%02X\n",j);
            i=pk[ik].code;
            if (i<0x100) keytab[j]=(keytab[j]&0xff00)|i;
            else keytab[j]=(keytab[j]&0xff)|i;
	    break;
        case KeyRelease:
            //j=myevent.xkey.keycode;
            //printf("%02X R\n",j);
            ik++;
            if (pk[ik].code==0xff) ik--;
            printf("Key/button> %s ",pk[ik].kname); fflush(stdout);
            break;
        case ButtonPress:

             switch(((XButtonEvent*)&myevent)->button){
             case 1: if (ik) ik--;
                     printf("\nKey/button> %s ",pk[ik].kname); fflush(stdout);
                     break;
             case 3: printf("Exiting\n");
                     fp=fopen(argv[1],"w");
                     for (i=0;i<256;i++) fprintf(fp,"%04X\n",keytab[i]);
                     fclose(fp);
                     exit(0);
             }

        }
    }
}

