/*************************************************************************

**************************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <linux/soundcard.h>

static int fd;	/* Descriptor del stream de las muestras de audio */

/**************************************************************************

Inicializacin de /dev/dsp a 44100 Hz, 8 bits, estreo

***************************************************************************/

void initdsp ()
{

	int rate=44100,channels=1,bits=8;
	
	if ((fd=open("/dev/dsp", O_WRONLY)) == -1) {
		fprintf(stderr, "Cannot open sound device");
		exit(0);
	}
	if (ioctl(fd, SOUND_PCM_WRITE_BITS, &bits) == -1) {
	    fprintf(stderr, "Cannot set %d bits/sample\n",bits);
	}
	if (ioctl(fd, SOUND_PCM_WRITE_CHANNELS, &channels) == -1) {
	    fprintf(stderr, "Cannot set %ld channel(s)\n",channels);
	}
	if (ioctl(fd, SOUND_PCM_WRITE_RATE, &rate) == -1) {
	    fprintf(stderr, "Cannot set rate %d Hz\n",rate);
	}
}

/**************************************************************************

Codificacin de un bit crudo en MANCHESTER

**************************************************************************/
static inline void wrrbit(unsigned char d)
{
	static unsigned char buf[54]={
	0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,
	255,255,255,255, 255,255,255,255, 255,255,255,255, 255,255,255,255, 255,255,
	0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,
	};
	
	if (d&1) write(fd,buf+18,36); else write(fd,buf,36);
}

/*************************************************************************

Codifificacin de un bit en NRZI

**************************************************************************/

static inline void wrbit(unsigned char d)
{
	static char ff;
	
	ff^=~d;	/* Conmutamos si el bit de datos es 0 */
	wrrbit(ff);
}

/*************************************************************************

Codificacin de un byte

**************************************************************************/

void wrbyte(unsigned char d)
{
	int i;
	for (i=0;i<8;i++) {
		wrbit(d>>7);
		d<<=1;
	}
}

/*************************************************************************

Escritura de un bloque (mximo 254 bytes)
retorna el n de bytes escritos.

**************************************************************************/
#define NCAB 10

int wrblock(char *pdat, int nbytes, int tipo)
{
	int i;

	for (i=0;i<NCAB; i++) wrbyte(0x01);	/* Preambulo */
	wrbyte(0x3c); wrbyte(0x5a);		/* Marca sincronismo */
	wrbyte(tipo);
	for(i=0;i<nbytes;i++) {
		wrbyte(*pdat++);
	}
	wrbyte(0); wrbyte(0);		/* Por si acacso*/
}

/**************************************************************************

Silencio entre bloques: parmetro: ms.

***************************************************************************/

void silencio (int ms)
{
	static unsigned char buf[44]={
	128,128,128,128, 128,128,128,128, 128,128,128,
	128,128,128,128, 128,128,128,128, 128,128,128,
	128,128,128,128, 128,128,128,128, 128,128,128,
	128,128,128,128, 128,128,128,128, 128,128,128
	};
	for (;ms>0;ms--) write(fd,buf,44);
}

main (int argc, char **argv)
{

	int i,fdi,tipo,tam;
	unsigned char buf[257];
	
	if (argc<2) {
		fprintf(stderr,"Usage: mo5_write file.k7\n");
		exit(0);
	}

	
	if ((fdi=open(argv[1],O_RDONLY))==-1) exit(0);
	
	initdsp();
	
	do {
		tipo=0;
		i=read(fdi,&tipo,1);	/* Tipo */
		if (!i) break;
		i=read(fdi,buf,1);	/* Longitud bloque */
		if (!i) break;
		tam=buf[0]; if (!tam) tam=256;

		printf("Tipo=0x%02X Tamao=0x%02X\n",tipo,tam);
		
		i=read(fdi,buf+1,tam-1);
		if (i!=tam-1) break;
		wrblock(buf, tam, tipo);	
		
		switch (tipo) {
		case 0:	silencio(500);
			break;
		case 1:	silencio(200);
			break;
		case 255:	silencio(2000);
			break;
		}
		
	} while (i);

	silencio(2000);	
	close(fdi);
	close(fd);
	exit(0);
}
